/**************************************************************************************************
  App.c
  Application.
  
  The GSM2 Click board is located in slot 3 of the MikroMedia+ for STM32 shield for this code.
  
  GSM2 Name        mikroBUS      Slot 3 Name      HDR1    HDR2      MM Schematic      STM32 Pin
---------------------------------------------------------------------------------------------------
  STAT (input)        AN         PB1/AN3           5                HDR-AN2           PB1   47
  PWRKEY(output)      RST        PB12              11               HDR-SPI2_SS       PB12  73
  RTS (output)        CS         PA15              17               HDR-SPI1_SS       PA15  110
  RI (input)          PWM        PD14/PWM3                 35       HDR-PWM2          PD14  85
  CTS (input)         INT        PE2/INT3                  41       HDR-INT2          PE2   1
  TXD (alt func)      TX         PA3/UART1-RX              47       HDR-UART1_RX      PA3   37
  RXD (alt func)      RX         PD5/UART1-TX              48       HDR-UART1_TX      PD5   119

**************************************************************************************************/

// Include Files
#include "App.h"
#include "Buffers.h"
#include "UART6.h"
#include "UART2.h"
#include "Timers.h"

// GSM2 click board connections
sbit GSM2_STAT at GPIOB_IDR.B1;
sbit GSM2_PWRKEY at GPIOB_ODR.B12;
sbit GSM2_RTS at GPIOA_ODR.B15;
sbit GSM2_RI at GPIOD_IDR.B14;
sbit GSM2_CTS at GPIOE_IDR.B2;


// Variables
char GSM2_State = 0;
char AppState = 0;
char ResponseBuf[256];
//short ResponseCode = -1;
unsigned short int TimerHandle;




extern char U2_MsgFlg;
extern char U2_RcvBuf[];



/**************************************************************************************************
  Initialize GSM2 */
void GSM2_Initialize(void) {
     
     GPIO_Clk_Enable(&GPIOA_BASE);                          // Enable clock on port A
     GPIO_Clk_Enable(&GPIOB_BASE);                          // Enable clock on port B
     GPIO_Clk_Enable(&GPIOD_BASE);                          // Enable clock on port D
     GPIO_Clk_Enable(&GPIOE_BASE);                          // Enable clock on port E

     GPIO_Digital_Input(&GPIOB_BASE,_GPIO_PINMASK_1);       // STAT is an input
     GPIO_Digital_Output(&GPIOB_BASE,_GPIO_PINMASK_12);     // PWRKEY is an output
     GPIO_Digital_Output(&GPIOA_BASE,_GPIO_PINMASK_15);     // RTS is an output
     GPIO_Digital_Input(&GPIOD_BASE,_GPIO_PINMASK_14);      // RI is an input
     GPIO_Digital_Input(&GPIOE_BASE,_GPIO_PINMASK_2);       // CTS is an input

     GSM2_RTS = 1;
}

/**************************************************************************************************
  Activate Power Key  */
void GSM2_KeyOn(void) {
     GSM2_PWRKEY = 1;
}

/**************************************************************************************************
  Deactivate Power Key  */
void GSM2_KeyOff(void) {
     GSM2_PWRKEY = 0;
}

/**************************************************************************************************
  Read Status  */
char GSM2_RdStat(void) {
     if (GSM2_STAT)
         return 1;
     else
         return 0;
}

/**************************************************************************************************
  Subroutine to set response code for returned messages */
short WaitForResponse(void) {
     char *BufNum;
     
     if (U2_MsgFlg) {
         BufNum = GetBuffer();
         strcpy(BufNum,U2_RcvBuf);
         U2_MsgFlg = 0;

         UART6_PutQueue(BufNum);

         if ((BufNum[0] == '\r') && (BufNum[1] == '\n')) {
              //UART6_PutQueue("Blank line received\r\n");
              return EMPTY;
         } else if ((BufNum[0] == 'O') && (BufNum[1] == 'K')) {
              //UART6_PutQueue("OK received\r\n");
              return OK;
         } else if ((BufNum[0] =='>') && (BufNum[1] == ' ')) {
              return PROMPT;
         }
     } else {
         return NEG;
     }
}

/**************************************************************************************************
  SubRoutine to set response code for returned messages */
void CheckTimeout(void) {
     if (check_timer(TimerHandle) == 0) {
         free_timer(TimerHandle);
         UART6_PutQueue("Timeout\r\n");
         GSM2_State = 0;
     }
}



/**************************************************************************************************
  General Purpose State Machine  */
void GSM2_TaskMgr(void) {
     char *BufNum;
     short ResponseCode;
     
     switch (GSM2_State) {

//****** Idle state *******************************************************************************
         case 0:
            ResponseCode = WaitForResponse();
            if (ResponseCode >= 0) {
                if (ResponseCode == EMPTY) {
                    //UART6_PutQueue("S0: Blank line received\r\n");
                } else if (ResponseCode == OK) {
                    //UART6_PutQueue("S0: OK received\r\n");
                }
            }
            break;



//****** Sequence to power up the GSM module ******************************************************
         case 1:   // Do hardware reset on GSM2 module
            GSM2_KeyOn();                   // Activate PWRKEY signal
            TimerHandle = get_timer();      // Get a free timer
            GSM2_State = 2;
            break;
         case 2:
            if (GSM2_RdStat()) {            // Wait for hardware signal
                set_timer(TimerHandle,100); // Delay an additional 100 ms
                GSM2_State = 3;
            }
            break;
         case 3:
            if (check_timer(TimerHandle) == 0) {
                free_timer(TimerHandle);    // Return timer to pool
                GSM2_KeyOff();              // Deactivate PWRKEY signal
                GSM2_State = 0;
            }
            break;



//****** Sequence to set the default baud rate of the GSM module **********************************
         case 6:   // Set fixed baud rate to 9600
            UART2_PutQueue("AT+IPR=9600\r\n");
            GSM2_State = 7;
            break;
         case 7:
            ResponseCode = WaitForResponse();
            if (ResponseCode == EMPTY) {
            }
            if (ResponseCode == OK) {
                GSM2_State = 8;
            }
            break;
         case 8:
            UART2_PutQueue("AT&W\r\n");
            GSM2_State = 9;
            break;
         case 9:
            ResponseCode = WaitForResponse();
            if (ResponseCode == EMPTY) {
            }
            if (ResponseCode == OK) {
                GSM2_State = 0;
            }
            break;


     
//****** Sequence to display the network parameters ***********************************************
         case 10:  // Query the network state information
            TimerHandle = get_timer();
            set_timer(TimerHandle,5000);         // Timeout in 5 seconds
            UART2_PutQueue("AT+CSQ\r\n");        // Query the signal strength of current network
            GSM2_State = 11;
            break;
         case 11:
            ResponseCode = WaitForResponse();
            if (ResponseCode == EMPTY) {         // Blank line
            
            } else if (ResponseCode == OK) {     // OK response
                set_timer(TimerHandle,5000);     // Reset timer
                UART2_PutQueue("AT+CREG?\r\n");  // Query register state of GSM network
                GSM2_State = 12;
            } else
                CheckTimeout();
            break;
         case 12:
            ResponseCode = WaitForResponse();
            if (ResponseCode == EMPTY) {         // Blank line
            
            } else if (ResponseCode == OK) {     // OK response
                set_timer(TimerHandle,5000);     // Reset timer
                UART2_PutQueue("AT+CGREG?\r\n"); // Query register state of GPRS network
                GSM2_State = 13;
            } else
                CheckTimeout();
            break;
         case 13:
            ResponseCode = WaitForResponse();
            if (ResponseCode == EMPTY) {         // Blank line
            
            } else if (ResponseCode == OK) {     // OK response
                set_timer(TimerHandle,5000);     // Reset timer
                UART2_PutQueue("AT+COPS?\r\n");  // Query the current selected operator
                GSM2_State = 14;
            } else
                CheckTimeout();
            break;
         case 14:
            ResponseCode = WaitForResponse();
            if (ResponseCode == EMPTY) {         // Blank line

            } else if (ResponseCode == OK) {     // OK response
                free_timer(TimerHandle);         // Return timer to pool
                GSM2_State = 0;
            } else
                CheckTimeout();
            break;



//****** Sequence to power down the GSM module ****************************************************
         case 20:   // Power down the module
            UART2_PutQueue("AT+QPOWD=1\r\n");
            GSM2_State = 21;
            break;
         case 21:
            ResponseCode = WaitForResponse();
            if (ResponseCode == EMPTY) {
            
            } else {
                GSM2_State = 0;
            }
            break;


     
//****** Sequence to send a text message to a phone number on the network *************************
         case 30:  // Send SMS in text mode
            TimerHandle = get_timer();
            set_timer(TimerHandle,5000);         // Timeout in 5 seconds
            UART2_PutQueue("AT+CMGF=1\r\n");     // Set text mode
            GSM2_State = 31;
            break;
         case 31:
            ResponseCode = WaitForResponse();
            if (ResponseCode == EMPTY) {         // Blank line

            } else if (ResponseCode == OK) {     // OK response
                set_timer(TimerHandle,5000);     // Reset timer
                UART2_PutQueue("AT+CSCS=\"GSM\"\r\n");  // Set character set to "GSM"
                GSM2_State = 33;  // Skip next state
            } else
                CheckTimeout();
            break;
// Disabled ---------------------------------------------------------------------------------------
         case 32:
            ResponseCode = WaitForResponse();
            if (ResponseCode == EMPTY) {         // Blank line

            } else if (ResponseCode == OK) {     // OK response
                set_timer(TimerHandle,5000);     // Reset timer
                UART2_PutQueue("ATE0\r\n");      // Disable echo
                GSM2_State = 33;
            } else
                CheckTimeout();
            break;
//-------------------------------------------------------------------------------------------------
         case 33:
            ResponseCode = WaitForResponse();
            if (ResponseCode == EMPTY) {         // Blank line

            } else if (ResponseCode == OK) {     // OK response
                set_timer(TimerHandle,5000);     // Reset timer
                UART2_PutQueue("AT+CMGS=\"+19052524261\"\r\n"); // Send SMS phone number
                GSM2_State = 34;
            } else
                CheckTimeout();
            break;
         case 34:
            ResponseCode = WaitForResponse();
            if (ResponseCode == EMPTY) {         // Blank line

            } else if (ResponseCode == PROMPT) { // OK response
                set_timer(TimerHandle,5000);     // Reset timer
                UART2_PutQueue("Hello world\r\n");
                GSM2_State = 35;
            } else
                CheckTimeout();
            break;
         case 35:
            ResponseCode = WaitForResponse();
            if (ResponseCode == EMPTY) {         // Blank line

            } else if (ResponseCode == PROMPT) { // OK response
                set_timer(TimerHandle,5000);     // Reset timer
                BufNum = GetBuffer();
                BufNum[0] = 0x1A;                // CTRL+Z
                BufNum[1] = 0x0D;                // CR
                BufNum[2] = 0x00;                // NULL
                //strcat(BufNum,"\r\n");
                UART2_PutQueue(BufNum);
                GSM2_State = 37; // Skip next state
            } else
                CheckTimeout();
            break;
// Disabled ---------------------------------------------------------------------------------------
         case 36:
            ResponseCode = WaitForResponse();
            if (ResponseCode == EMPTY) {         // Blank line

            } else if (ResponseCode == OK) {     // OK response
                set_timer(TimerHandle,5000);     // Reset timer
                UART2_PutQueue("ATE1\r\n");      // Enable echo
                GSM2_State = 37;
            } else
                CheckTimeout();
            break;
//-------------------------------------------------------------------------------------------------
         case 37:
            ResponseCode = WaitForResponse();
            if (ResponseCode == EMPTY) {         // Blank line

            } else if (ResponseCode == OK) {     // OK response
                free_timer(TimerHandle);         // Return timer to pool
                GSM2_State = 0;
            } else
                CheckTimeout();
            break;

     
     
//****** Sequence to delete messages 1, 2, and 3 **************************************************
         case 40:  // Delete SMS messages 1, 2, and 3
            TimerHandle = get_timer();
            set_timer(TimerHandle,5000);         // Timeout in 5 seconds
            UART2_PutQueue("AT+CMGF=1\r\n");     // Set text mode
            GSM2_State = 41;
            break;
         case 41:
            ResponseCode = WaitForResponse();
            if (ResponseCode == EMPTY) {         // Blank line

            } else if (ResponseCode == OK) {     // OK response
                set_timer(TimerHandle,5000);     // Reset timer
                UART2_PutQueue("AT+CSCS=\"GSM\"\r\n");  // Set character set to "GSM"
                GSM2_State = 42;
            } else
                CheckTimeout();
            break;
         case 42:
            ResponseCode = WaitForResponse();
            if (ResponseCode == EMPTY) {         // Blank line

            } else if (ResponseCode == OK) {     // OK response
                set_timer(TimerHandle,5000);     // Reset timer
                UART2_PutQueue("AT+CMGD=1\r\n");     // Delete SMS message 1
                GSM2_State = 43;
            } else
                CheckTimeout();
            break;
         case 43:
            ResponseCode = WaitForResponse();
            if (ResponseCode == EMPTY) {         // Blank line

            } else if (ResponseCode == OK) {     // OK response
                set_timer(TimerHandle,5000);     // Reset timer
                UART2_PutQueue("AT+CMGD=2\r\n");     // Delete SMS message 2
                GSM2_State = 44;
            } else
                CheckTimeout();
            break;
         case 44:
            ResponseCode = WaitForResponse();
            if (ResponseCode == EMPTY) {         // Blank line

            } else if (ResponseCode == OK) {     // OK response
                set_timer(TimerHandle,5000);     // Reset timer
                UART2_PutQueue("AT+CMGD=3\r\n");     // Delete SMS message 3
                GSM2_State = 45;
            } else
                CheckTimeout();
            break;
         case 45:
            ResponseCode = WaitForResponse();
            if (ResponseCode == EMPTY) {         // Blank line

            } else if (ResponseCode == OK) {     // OK response
                free_timer(TimerHandle);         // Return timer to pool
                GSM2_State = 0;
            } else
                CheckTimeout();
            break;
     }
}



/**************************************************************************************************
  Given a string s that contains tokens delimited by a comma, this function strips the first
  token from the string and puts it in t. The return code is -1 if the string is empty, 0 if there
  are no more tokens following this one, 1 if there are more tokens to process. String s is
  destroyed in the process.   */
short GetToken(char *s, char *t) {
     char *p;                       // Token pointer

     if (strlen(s) == 0) {          // If the string is empty
         return (-1);
     }
     p = strchr(s,',');             // Find location of comma
     if (p == NUL) {                // If didn't find a comma
         strcpy(t,s);               // Copy the final token
         strcpy(s,"");              // Nullify the input string
         return 0;
     }
     s[strlen(s)-strlen(p)] = '\0'; // Replace the comma with a null
     strcpy(t,s);                   // Copy the token
     p++;                           // Advance the pointer to the next character position
     strcpy(s,p);                   // Overlay the input string minus the first token
     return 1;
}