/**************************************************************************************************
  UART2.c
  This UART module communicates with the GSM2 Click board on the MikroMedia+ for STM32 shield.

  NOTE: ERRORS IN MIKROELEKTRONIKA SCHEMATIC!!!!!!!
  Net labeled HDR-UART1_TX connected to port PD5 physical pin 119 is actually USART2_TX
  Net labeled HDR-UART1_RX connected to port PA3 physical pin  37 is actually USART2_RX

  USART2 is on APB1 BUS and the peripheral clock is 37.5 MHz
  USARTDIV = Fck / 16 * baud = 37,500,000 / 153,600 = 0d244.140625
  DIV_Mantissa = 0d244 = oxF4
  DIV_Fraction = 16 * 0d0.140625 = 0d2.25  approximately 0x2
  Therefore USART6_BRR = 0x0F42
**************************************************************************************************/

// Include Files
#include "UART2.h"
#include "UART6.h"
#include "Buffers.h"

#pragma funcall UART2_Rx1, UART2_Rx2

// Variables
char U2_RxIn;                    // Input pointer to the next byte in receive FIFO
char U2_RxOut;                   // Output pointer to the next byte in receive FIFO
char U2_RxQueue[U2_rxqsize];     // Re-circulating receive FIFO

char U2_RcvBuf[U2_RcvBufSize];   // Receive messages are assembled in this buffer
char U2_RcvBufPtr;               // Pointer to the next free byte in RcvBuf


char U2_TxIn;                    // Input pointer next element in the transmit FIFO
char U2_TxOut;                   // Output pointer to the next element in the transmit FIFO
char *U2_TxQueue[U2_txqsize];    // Queue of message pointers in the transmit FIFO

char *U2_XmtBuf;                 // Pointer to the current message being transmitted
char U2_XmtBufPtr;               // Points to a byte in transmit buffer


// The receive message routine has 2 states
tPointerFcn UART2_RxFuncPtr = UART2_Rx1;  // Start in State 1
char U2_MsgFlg = 0;              // Flag indicating that a message is in receive buffer
char PromptFlag = 0;



/*************************************************************************************************/
// Hardware Level Functions
/**************************************************************************************************
  Interrupt Handler */
void UART2_Interrupt() iv IVT_INT_USART2 ics ICS_AUTO {
     char R_data;        // Receive data byte
     char T_data;        // Transmit data byte


     if ((RXNE_USART2_SR_bit) /* || (ORE_bit) */ ) {
         R_data = USART2_DR;                 // Fetch a byte from receive register
         U2_RxQueue[U2_RxIn++] = R_data;     // Put data byte in re-circulating buffer, increment pointer
         if (U2_RxIn > (U2_rxqsize-1))       // Adjust pointer if necessary
             U2_RxIn = 0;
     }

     if ((TXE_USART2_SR_bit)&&(TXEIE_USART2_CR1_bit)) {
         T_data = U2_XmtBuf[U2_XmtBufPtr++]; // Get a byte from the transmit buffer
         if (T_data == NUL) {                // If it is the end of message marker
             TXEIE_USART2_CR1_bit = 0;       // Disable transmitter interrupts
         } else {
             USART2_DR = T_data;             // Transfer data byte to transmit register
         }
     }
}

/**************************************************************************************************
  This routine sets the baud rate. */
void UART2_SetBaud(void) {

     // USART2 TX is on port D bit 5, and USART2 RX is on port A bit 3
     
     GPIOAEN_bit = 1;                  // Set GPIOA clock enable bit
     GPIODEN_bit = 1;                  // Set GPIOD clock enable bit

     GPIOA_MODER |= 0x00000080;        // Set port A MODER bit 3 to 10: alternate function mode
     GPIOD_MODER |= 0x00000800;        // Set port D MODER bit 5 to 10: alternate function mode
     
     GPIOA_AFRL |= 0x00007000;         // Select alternate function 0111: AF7 for port A bit 3
     GPIOD_AFRL |= 0x00700000;         // Select alternate function 0111: AF7 for port D bit 35
     
     USART2EN_bit = 1;                 // Set USART2 clock enable bit
     USART2_CR1 = 0x200C;              // Set UE, TE, and RE in USART2 control register 1
     USART2_BRR = 0x0F42;              // Set baud rate register for 9600

     NVIC_IntEnable(IVT_INT_USART2);   // Eneble interrupt vector
}

/**************************************************************************************************
  This routine starts the receiver. */
void UART2_StartRx(void) {
     RXNEIE_USART2_CR1_bit = 1;        // Interrupt when receive buffer is full (RXNE=1)
}

/**************************************************************************************************
  This routine starts the transmitter. */
void UART2_StartTx(void) {
     TXEIE_USART2_CR1_bit = 1;         // Interrupt when transmit buffer is empty (TXE=1)
}
//-------------------------------------------------------------------------------------------------



/*************************************************************************************************/
// UART6 Functions
/**************************************************************************************************
  Initialize UART */
void UART2_Initialize(void) {
     UART2_RxQueueInit();
     UART2_TxQueueInit();

     UART2_SetBaud();
     UART2_StartRx();
}

// The receive portion ****************************************************************************
/**************************************************************************************************
  This routine initializes the receive buffer in and out pointers. */
void UART2_RxQueueInit(void) {
     U2_RxIn = U2_RxOut = 0;           // Set pointers to 0
     U2_RcvBufPtr = 0;
}

/**************************************************************************************************
  This routine retrieves a byte from the re-circulating buffer if present. Returns 0 if no data
  available. Returns 1 if a data byte has been transferred successfully. The pointer to the data
  byte is passed to the routine. */
char UART2_GetChar(char *rxbyte) {
     if (U2_RxIn != U2_RxOut) {              // If there is a character in the buffer
        *rxbyte = U2_RxQueue[U2_RxOut++];    // Extract the character and update the pointer
        if (U2_RxOut > (U2_rxqsize-1))       // Check for wrap
           U2_RxOut = 0;
        return 1;
     }else{
        return 0;                            // No data available
     }
}

/**************************************************************************************************
  UART_Rx State Machine. Call this function in the main loop. */
void UART2_Rx(void) {
     UART2_RxFuncPtr();      // Use the function pointer to go to a state
}

/**************************************************************************************************
  UART_Rx State 1: Accumulate received characters in a buffer until a Line Feed character is
  received then insert a NUL character into the buffer and go to State 2. */
void UART2_Rx1(void){
     char rxchar;

     if (UART2_GetChar(&rxchar)) {
         switch (rxchar) {
             case NUL:       // Discard NUL character
                 break;

             case '\r':
                 U2_RcvBuf[U2_RcvBufPtr++] = rxchar;       // Put return character in buffer
                 break;

             case '\n':
                 U2_RcvBuf[U2_RcvBufPtr++] = rxchar;       // Put new line character into buffer
                 U2_RcvBuf[U2_RcvBufPtr] = 0;              // Put end marker into buffer
                 U2_RcvBufPtr = 0;                         // Point to start of buffer
                 
                 U2_MsgFlg = 1;                            // Indicate message available
                 
                 UART2_RxFuncPtr = UART2_Rx2;              // Go to state 2
                 break;

             case '>':
                 U2_RcvBuf[U2_RcvBufPtr++] = rxchar;
                 PromptFlag = 1;
                 break;
                 
             case ' ':
                 U2_RcvBuf[U2_RcvBufPtr++] = rxchar;
                 if (PromptFlag == 1) {
                     U2_RcvBuf[U2_RcvBufPtr] = 0;               // Put end marker into buffer
                     U2_RcvBufPtr = 0;                          // Point to start of buffer
                     U2_MsgFlg = 1;                             // Indicate message available
                 }
                 PromptFlag = 0;                                // Clear prompt flag regardless
                 break;

             default:
                 if ((rxchar > 31) && (rxchar < 127)) {         // Use only ASCII set
                      if (U2_RcvBufPtr < (U2_RcvBufSize-2))     // Prevents buffer overrun
                          U2_RcvBuf[U2_RcvBufPtr++] = rxchar;   // Put character in buffer
                 }
                 break;
         }
     }
}

/**************************************************************************************************
  UART_Rx State 2: Wait for someone to release the receive buffer then return to State 1. */
void UART2_Rx2(void) {
     if (U2_MsgFlg == 0) {
         UART2_RxFuncPtr = UART2_Rx1;  // Go back to state 1
     }
}



// The transmit portion ***************************************************************************
/**************************************************************************************************
  This routine initializes the transmit buffer queue in and out pointers. */
void UART2_TxQueueInit(void) {
     U2_TxIn = U2_TxOut = 0;           // Set pointers to 0
     U2_XmtBufPtr = 0;
}

/**************************************************************************************************
  This routine makes an entry into the transmit queue of pointers and updates the in pointer to
  the next available location. */
void UART2_PutQueue(char *msg) {
     U2_TxQueue[U2_TxIn++] = msg;      // Put the message pointer into FIFO
     if (U2_TxIn > (U2_txqsize-1))     // Adjust FIFO in pointer
         U2_TxIn = 0;
}

/**************************************************************************************************
  This routine enters a pointer to a text message and starts the transmitter. */
void UART2_SendMsg(char *s) {
     U2_XmtBuf = s;                    // Pointer to the message for the interrupt handler
     U2_XmtBufPtr = 0;                 // Set character counter to start of text message
     UART2_StartTx();                  // Turn on the transmitter
}

/**************************************************************************************************
  This routine checks the transmit buffer queue and if it is not empty, it extracts the pointer
  to the message and sets up the transmitter. Call this function in the main loop. */
void UART2_CheckQueue(void) {
     if (TXEIE_USART2_CR1_bit == 0) {  // Don't try to start transmitter if already transmitting
         if (U2_TxIn != U2_TxOut) {                 // If pointers are not the same
             UART2_SendMsg(U2_TxQueue[U2_TxOut++]); // Hand off the message pointer and increment counter
             if (U2_TxOut > (U2_txqsize-1))         // Check for wrap
                 U2_TxOut = 0;
         }
     }
}